#!/usr/bin/perl  -w

#
# floppyfwcreate.pl
# Made by Thomas Lundquist <thomasez@zelow.no>
# started 2003-08-01
#
# 0.0.6

use strict;

#
# From the config file:
#
use vars qw($imagelocation $pppoeimage $standardimage $tmpimage $config
		$floppydevice);

#
# A global variable: 
#
my (%newconfig);
my (%text);


my ($rundir) = $1 if ($0 =~ /(.*)\/floppyfwcreate.pl/);
require("$rundir/floppyfwcreate.conf");

#
# Pick a language, interesting 
#

my ($lang) = $ENV{LC_ALL} || $ENV{LC_MESSAGES} || $ENV{LANG};

print "Lang:$lang\n";

my $pofile = "en_UK.po";

$pofile = "nb_NO.po" if ($lang =~ /NO/i);

&readPoFile("$rundir/$pofile");

unless (-f "$rundir/$pppoeimage") {
	&grabImage($imagelocation, $pppoeimage, $tmpimage);
	&dumpImage($tmpimage, $floppydevice);
} else {
	&dumpImage("$rundir/$pppoeimage", $floppydevice);
}

&mountFloppy($floppydevice);
&askConfig($config);
# &editConfig("/tmp/config", %newconfig);
&editConfig("/floppy/config", %newconfig);
&umountFloppy($floppydevice);

sub askConfig {

	my ($trav) = shift;

	#
	# This thingie wil traverse the (enormous) config hash and ask 
	# where it's set to ask or just throw the 
	#
	
	foreach my $section (@{$trav->{'order'}}) {
		if ($trav->{$section}->{'order'}) {
			&askConfig($trav->{$section}) ;
			next;
		}

		my ($value);


		#
		# Are we going to ask this question?
		#
		if ($trav->{$section}->{'ask'}) {
			print grabtext($trav->{$section}->{'description'}) 
				|| $section;
			print " [" . join (", ", @{$trav->{$section}->{'alternatives'}}) . "]"
				if ($trav->{$section}->{'alternatives'});
			print ": ";
			$value = <>;
			chop $value;
		}

		&askConfig($trav->{$section}->{$value}) 
			if ($value && exists($trav->{$section}->{$value}));

		#
		# Garbage in garbage out!
		# (And yes, it should be a lot better, like check if it's
		# one of the alternatrives and so on)
		#
		$newconfig{$section} = $value 
			|| $trav->{$section}->{'default'};

	}

}

sub editConfig {
	my ($filename, %newconfig) = @_;

	my ($content);

	open (IN, $filename);

	while (<IN>) {
		my ($key);

		if (($key) = /(^\w+)=/) {
			s/^$key.*/$key=$newconfig{$key}/ if ($newconfig{$key});
		}
		$content .= $_;
	}
	close (IN);

	foreach my $key (sort(keys(%newconfig))) {

	}

	open (CONF, "+>$filename");
#	print CONF "#This file was automagically edited by floppyfwcreate.pl\n";
	print CONF $content;
	close (CONF);

}

sub grabImage {
	my ($location) = shift;
	my ($image) = shift;
	my ($tmp) = shift;

	# This code nicked from 
	# <URL:http://jspot.org/jeff/crs/programming/talks/LWP_talk_for_perl_mongers/12.wimpy>
	# (and it does need some fixing for this useage (error_as_HTML at least)

	print "Grabbing $imagelocation/$image.\n";
	use LWP::UserAgent;
	use HTTP::Request;
	use HTTP::Response;
	my $ua = new LWP::UserAgent;
	my $request = new HTTP::Request('GET', "$imagelocation/$image");
	my $response = $ua->request($request);

	if ($response->is_success) {
		open (IMAGE, "+>$tmp");
		print IMAGE $response->content;
		close (IMAGE);
		return 1;
	} else {
		print $response->error_as_HTML;
		return 0;
	}

}

sub dumpImage {
	my ($image) = shift;
	my ($floppydevice) = shift;

	# 
	# Ugly hack for now.
	# 
	print "Dumping $image to $floppydevice\n";
	open (FD, "+>$floppydevice");
	open (IMG, $image);
	while (<IMG>) {
		print FD $_;
	}
	close (IMG);
	close (FD);

}

sub mountFloppy {
	my ($floppydevice) = shift;

	# 
	# I'd like to use a perl internal function here.
	# 

	# system("/bin/mount", "-t vfat", $floppydevice, "/floppy");
	system("/bin/mount -t vfat $floppydevice /floppy");

}

sub umountFloppy {
	my ($floppydevice) = shift;

	# 
	# I'd like to use a perl internal function here.
	# 

	system("/bin/umount", "/floppy");

}

sub readPoFile {
	my ($file) = shift;

	my ($key);

	open (TXT, $file);
	while (<TXT>) {
		next if (/^$/);
		$key = $1 if (/^msgid "(\w+)/);
		$text{$key} = $1 if (/^msgstr "(.*)"/);
	}
	close (TXT);

}

#
# Using "grabtext" since "gettext" may intefere with gettext..
# 
sub grabtext {
	my ($key) = shift;
	return $text{$key};
}
